@@ -1,9 +1,128 @@
-from mycroft_bus_client import MessageBusClient, Message
+from mycroft_bus_client import MessageBusClient
+from mycroft_bus_client.message import dig_for_message, Message
 from ovos_utils.log import LOG
-from ovos_utils.configuration import read_mycroft_config
+from ovos_utils.configuration import read_mycroft_config, get_default_lang
 from ovos_utils import create_loop
+from ovos_utils.json_helper import merge_dict
 import time
 import json
+from pyee import BaseEventEmitter
+from threading import Event
+
+
+class FakeBus:
+    def __init__(self, *args, **kwargs):
+        self.started_running = False
+        self.ee = kwargs.get("emitter") or BaseEventEmitter()
+        self.ee.on("error", self.on_error)
+        self.on_open()
+
+    def on(self, msg_type, handler):
+        self.ee.on(msg_type, handler)
+
+    def once(self, msg_type, handler):
+        self.ee.once(msg_type, handler)
+
+    def emit(self, message):
+        self.ee.emit("message", message.serialize())
+        self.ee.emit(message.msg_type, message)
+
+    def wait_for_message(self, message_type, timeout=3.0):
+        """Wait for a message of a specific type.
+
+        Arguments:
+            message_type (str): the message type of the expected message
+            timeout: seconds to wait before timeout, defaults to 3
+
+        Returns:
+            The received message or None if the response timed out
+        """
+        received_event = Event()
+        received_event.clear()
+
+        msg = None
+
+        def rcv(m):
+            nonlocal msg
+            msg = m
+            received_event.set()
+
+        self.ee.once(message_type, rcv)
+        received_event.wait(timeout)
+        return msg
+
+    def wait_for_response(self, message, reply_type=None, timeout=3.0):
+        """Send a message and wait for a response.
+
+        Arguments:
+            message (Message): message to send
+            reply_type (str): the message type of the expected reply.
+                              Defaults to "<message.msg_type>.response".
+            timeout: seconds to wait before timeout, defaults to 3
+
+        Returns:
+            The received message or None if the response timed out
+        """
+        reply_type = reply_type or message.msg_type + ".response"
+        received_event = Event()
+        received_event.clear()
+
+        msg = None
+
+        def rcv(m):
+            nonlocal msg
+            msg = m
+            received_event.set()
+
+        self.ee.once(reply_type, rcv)
+        self.emit(message)
+        received_event.wait(timeout)
+        return msg
+
+    def remove(self, msg_type, handler):
+        try:
+            self.ee.remove_listener(msg_type, handler)
+        except:
+            pass
+
+    def remove_all_listeners(self, event_name):
+        self.ee.remove_all_listeners(event_name)
+
+    def create_client(self):
+        return self
+
+    def on_error(self, error):
+        LOG.error(error)
+
+    def on_open(self):
+        pass
+
+    def on_close(self):
+        pass
+
+    def run_forever(self):
+        self.started_running = True
+
+    def run_in_thread(self):
+        self.run_forever()
+
+    def close(self):
+        self.on_close()
+
+
+def get_message_lang(message=None):
+    """Get the language from the message or the default language.
+    Args:
+        message: message to check for language code.
+    Returns:
+        The language code from the message or the default language.
+    """
+    message = message or dig_for_message()
+    default_lang = get_default_lang()
+    if not message:
+        return default_lang
+    lang = message.data.get("lang") or message.context.get("lang") or default_lang
+    return lang.lower()
 
 
 def get_websocket(host, port, route='/', ssl=False, threaded=True):
@@ -50,7 +169,7 @@ def listen_once_for_message(msg_type, handler, bus=None):
     return bus
 
 
-def wait_for_reply(message, reply_type=None, timeout=None, bus=None):
+def wait_for_reply(message, reply_type=None, timeout=3.0, bus=None):
     """Send a message and wait for a response.
 
     Args:
@@ -104,6 +223,43 @@ def send_message(message, data=None, context=None, bus=None):
         bus.close()
 
 
+def send_binary_data_message(binary_data, msg_type="mycroft.binary.data",
+                             msg_data=None, msg_context=None, bus=None):
+    msg_data = msg_data or {}
+    msg = {
+        "type": msg_type,
+        "data": merge_dict(msg_data, {"binary": binary_data.hex()}),
+        "context": msg_context or None
+    }
+    send_message(msg, bus=bus)
+
+
+def send_binary_file_message(filepath, msg_type="mycroft.binary.file",
+                             msg_context=None, bus=None):
+    with open(filepath, 'rb') as f:
+        binary_data = f.read()
+    msg_data = {"path": filepath}
+    send_binary_data_message(binary_data, msg_type=msg_type, msg_data=msg_data,
+                             msg_context=msg_context, bus=bus)
+
+
+def decode_binary_message(message):
+    if isinstance(message, str):
+        try:  # json string
+            message = json.loads(message)
+            binary_data = message.get("binary") or message["data"]["binary"]
+        except:  # hex string
+            binary_data = message
+    elif isinstance(message, dict):
+        # data field or serialized message
+        binary_data = message.get("binary") or message["data"]["binary"]
+    else:
+        # message object
+        binary_data = message.data["binary"]
+    # decode hex string
+    return bytearray.fromhex(binary_data)
+
+
 class BusService:
     """
     Provide some service over the messagebus for other components
@@ -165,7 +321,7 @@ class BusFeedProvider:
 
     """
 
-    def __init__(self, trigger_message, name=None, bus=None):
+    def __init__(self, trigger_message, name=None, bus=None, config=None):
         """
            initialize responder
 
@@ -173,13 +329,14 @@ class BusFeedProvider:
                 name(str): name identifier for .conf settings
                 bus (WebsocketClient): mycroft messagebus websocket
         """
+        config = config or read_mycroft_config()
         self.trigger_message = trigger_message
         self.name = name or self.__class__.__name__
         self.bus = bus or get_mycroft_bus()
         self.callback = None
         self.service = None
         self._daemon = None
-        self.config = read_mycroft_config().get(self.name, {})
+        self.config = config.get(self.name, {})
 
     def update(self, data):
         """
@@ -308,12 +465,14 @@ class BusFeedConsumer:
 
     """
 
-    def __init__(self, query_message, name=None, timeout=5, bus=None):
+    def __init__(self, query_message, name=None, timeout=5, bus=None,
+                 config=None):
         self.query_message = query_message
         self.query_message.context["source"] = self.name
         self.name = name or self.__class__.__name__
         self.bus = bus or get_mycroft_bus()
-        self.config = read_mycroft_config().get(self.name, {})
+        config = config or read_mycroft_config()
+        self.config = config.get(self.name, {})
         self.timeout = timeout
         self.query = None
         self.valid_responses = []
