@@ -1,26 +1,122 @@
-#!/usr/bin/env python3
-import sys
+import inspect
 import os
-import subprocess
 import re
 import shutil
+import subprocess
+import sys
+import sysconfig
 from enum import Enum
-import platform
-import socket
-from os.path import expanduser
+from os.path import expanduser, exists, join
+
+from ovos_utils.log import LOG
 
 
 class MycroftRootLocations(str, Enum):
     PICROFT = "/home/pi/mycroft-core"
     BIGSCREEN = "/home/mycroft/mycroft-core"
-    OVOS = "/home/mycroft/mycroft-core"  # TODO ovos here
-    OLD_MARK1 = "/opt/venvs/mycroft-core/lib/python3.4/site-packages/"
-    MARK1 = "/opt/venvs/mycroft-core/lib/python3.7/site-packages/"
-    MARK2 = "/home/mycroft/mycroft-core"  # TODO mark2 here
-    HOME = expanduser("~/mycroft-core")
+    OVOS = "/usr/lib/python3.9/site-packages"
+    OLD_MARK1 = "/opt/venvs/mycroft-core/lib/python3.4/site-packages"
+    MARK1 = "/opt/venvs/mycroft-core/lib/python3.7/site-packages"
+    MARK2 = "/opt/mycroft"
+    HOME = expanduser("~/mycroft-core")  # git clones
+
+
+_USER_DEFINED_ROOT = None
+
+
+def is_running_from_module(module_name):
+    # Stack:
+    # [0] - _log()
+    # [1] - debug(), info(), warning(), or error()
+    # [2] - caller
+    stack = inspect.stack()
+
+    # Record:
+    # [0] - frame object
+    # [1] - filename
+    # [2] - line number
+    # [3] - function
+    # ...
+    for record in stack[2:]:
+        mod = inspect.getmodule(record[0])
+        name = mod.__name__ if mod else ''
+        # module name in file path of caller
+        # or import name matches module name
+        if f"/{module_name}/" in record[1] or \
+                name.startswith(module_name.replace("-", "_").replace(" ", "_")):
+            return True
+    return False
+
+
+# system utils
+def ntp_sync():
+    # Force the system clock to synchronize with internet time servers
+    subprocess.call('service ntp stop', shell=True)
+    subprocess.call('ntpd -gq', shell=True)
+    subprocess.call('service ntp start', shell=True)
+
+
+def system_shutdown():
+    # Turn the system completely off (with no option to inhibit it)
+    subprocess.call('sudo systemctl poweroff -i', shell=True)
+
+
+def system_reboot():
+    # Shut down and restart the system
+    subprocess.call('sudo systemctl reboot -i', shell=True)
+
+
+def ssh_enable():
+    # Permanently allow SSH access
+    subprocess.call('sudo systemctl enable ssh.service', shell=True)
+    subprocess.call('sudo systemctl start ssh.service', shell=True)
+
+
+def ssh_disable():
+    # Permanently block SSH access from the outside
+    subprocess.call('sudo systemctl stop ssh.service', shell=True)
+    subprocess.call('sudo systemctl disable ssh.service', shell=True)
+
+
+# platform fingerprinting
+def set_root_path(path):
+    global _USER_DEFINED_ROOT
+    _USER_DEFINED_ROOT = path
+    LOG.info(f"mycroft root set to {path}")
+
+
+def find_root_from_sys_path():
+    """Find mycroft root folder from sys.path, eg. venv site-packages."""
+    for p in [path for path in sys.path if path != '']:
+        if exists(join(p, 'mycroft', 'configuration', 'mycroft.conf')):
+            return p
+    else:
+        return None
+
+
+def find_root_from_sitepackages():
+    """Find root from system or venv's sitepackages."""
+    site = sysconfig.get_paths()['platlib']
+    if exists(join(site, 'mycroft', 'configuration', 'mycroft.conf')):
+        return site
+    else:
+        return None
 
 
 def search_mycroft_core_location():
+    """Check python path (.venv), system packages and finally known mycroft
+    locations."""
+    # downstream wants to override the root location
+    if _USER_DEFINED_ROOT:
+        return _USER_DEFINED_ROOT
+    # if we are in a .venv that should take precedence over everything else
+    if find_root_from_sitepackages():
+        return find_root_from_sitepackages()
+    # if there is a system wide install that should take precedence over
+    # hardcoded locations
+    elif find_root_from_sys_path():
+        return find_root_from_sys_path()
+    # finally look at default locations
     for p in MycroftRootLocations:
         if os.path.isdir(p):
             return p
@@ -40,13 +136,16 @@ def get_desktop_environment():
         desktop_session = os.environ.get("DESKTOP_SESSION")
         if desktop_session is not None:  # easier to match if we doesn't have  to deal with character cases
             desktop_session = desktop_session.lower()
-            if desktop_session in ["gnome", "unity", "cinnamon", "mate", "xfce4", "lxde", "fluxbox",
-                                   "blackbox", "openbox", "icewm", "jwm", "afterstep", "trinity", "kde"]:
+            if desktop_session in ["gnome", "unity", "cinnamon", "mate",
+                                   "xfce4", "lxde", "fluxbox",
+                                   "blackbox", "openbox", "icewm", "jwm",
+                                   "afterstep", "trinity", "kde"]:
                 return desktop_session
             # Special cases
             # Canonical sets $DESKTOP_SESSION to Lubuntu rather than LXDE if using LXDE.
             # There is no guarantee that they will not do the same with the other desktop environments.
-            elif "xfce" in desktop_session or desktop_session.startswith("xubuntu"):
+            elif "xfce" in desktop_session or desktop_session.startswith(
+                    "xubuntu"):
                 return "xfce4"
             elif desktop_session.startswith("ubuntu"):
                 return "unity"
@@ -100,8 +199,18 @@ def is_installed(executable):
 
 def has_screen():
     have_display = "DISPLAY" in os.environ
+    # X server not running
+    if not have_display:
+        # raspberry pi specific check
+        try:
+            have_display = b"device_name=" in subprocess.check_output("tvservice -n 2>&1", shell=True)
+        except Exception as e:
+            pass
+        
+    # fallback check using matplotlib if available
+    # seems to be foolproof and OS agnostic 
+    # but do not want to drag the dependency
     if not have_display:
-        # fallback check using matplotlib if available
         try:
             import matplotlib.pyplot as plt
             try:
@@ -112,51 +221,3 @@ def has_screen():
         except ImportError:
             pass
     return have_display
-
-
-def get_platform_fingerprint():
-    return {
-        "hostname": socket.gethostname(),
-        "platform": platform.platform(),
-        "python_version": platform.python_version(),
-        "system": platform.system(),
-        "version": platform.version(),
-        "arch": platform.machine(),
-        "release": platform.release(),
-        "desktop_env": get_desktop_environment(),
-        "mycroft_core_location": search_mycroft_core_location(),
-        "can_display": has_screen(),
-        "is_gui_installed": is_installed("mycroft-gui-app"),
-        "is_vlc_installed": is_installed("vlc"),
-        "pulseaudio_running": is_process_running("pulseaudio")
-    }
-
-
-def ntp_sync():
-    # Force the system clock to synchronize with internet time servers
-    subprocess.call('service ntp stop', shell=True)
-    subprocess.call('ntpd -gq', shell=True)
-    subprocess.call('service ntp start', shell=True)
-
-
-def system_shutdown():
-    # Turn the system completely off (with no option to inhibit it)
-    subprocess.call('sudo systemctl poweroff -i', shell=True)
-
-
-def system_reboot():
-    # Shut down and restart the system
-    subprocess.call('sudo systemctl reboot -i', shell=True)
-
-
-def ssh_enable():
-    # Permanently allow SSH access
-    subprocess.call('sudo systemctl enable ssh.service', shell=True)
-    subprocess.call('sudo systemctl start ssh.service', shell=True)
-
-
-def ssh_disable():
-    # Permanently block SSH access from the outside
-    subprocess.call('sudo systemctl stop ssh.service', shell=True)
-    subprocess.call('sudo systemctl disable ssh.service', shell=True)
-
